from typing import List

from statham.dsl.constants import Maybe
from statham.dsl.elements import Array, Integer, Number, Object, String
from statham.dsl.property import Property


class NestedSchema(Object):

    id: str = Property(
        String(
            format="uuid",
            pattern="^[0-9a-fA-F]{8}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{12}$",
        ),
        required=True,
    )

    timestamp: Maybe[str] = Property(
        String(
            format="date-time",
            pattern="^(-?(?:[1-9][0-9]*)?[0-9]{4})-(1[0-2]|0[1-9])-(3[01]|0[1-9]|[12][0-9])T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-](?:2[0-3]|[01][0-9]):[0-5][0-9])?$",
        )
    )

    version: int = Property(Integer(default=0))

    annotation: str = Property(String(default="unannotated"))


class SimpleSchema(Object):

    related: NestedSchema = Property(NestedSchema, required=True)

    amount: Maybe[float] = Property(Number())

    children: Maybe[List[NestedSchema]] = Property(Array(NestedSchema))



from typing import Any, Union

from statham.dsl.elements import AnyOf, Integer, Null, Number, Object, String
from statham.dsl.property import Property


class Model(Object):

    number_integer: Union[float, int] = Property(
        AnyOf(
            Number(minimum=2.5, exclusiveMaximum=5),
            Integer(minimum=2.5, exclusiveMaximum=5),
            default=3.5,
        )
    )

    string_integer: Union[str, int] = Property(
        AnyOf(
            String(format="date-time"),
            Integer(minimum=1970, exclusiveMaximum=2050),
            default=2000,
        )
    )

    string_null: Union[str, None] = Property(
        AnyOf(
            String(
                pattern="^[0-9a-fA-F]{8}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{4}\\-[0-9a-fA-F]{12}$"
            ),
            Null(),
            default=None,
        )
    )



from typing import Any, List, Union

from statham.dsl.constants import Maybe
from statham.dsl.elements import AnyOf, Array, Integer, Object, String
from statham.dsl.property import Property


class ListOfStringsItem(Object, additionalProperties=False):

    string_property: Maybe[str] = Property(String())


class ListOfIntegersItem(Object, additionalProperties=False):

    integer_property: Maybe[int] = Property(Integer())


class ListAnyOfItem0(Object, additionalProperties=False):

    string_prop: Maybe[str] = Property(String())


class ListAnyOfItem1(Object, additionalProperties=False):

    integer_prop: Maybe[int] = Property(Integer())


class Autoname(Object):

    list_of_strings: Maybe[List[ListOfStringsItem]] = Property(
        Array(ListOfStringsItem)
    )

    list_of_integers: Maybe[List[ListOfIntegersItem]] = Property(
        Array(ListOfIntegersItem)
    )

    list_any_of: Maybe[
        List[Union[ListAnyOfItem0, ListAnyOfItem1, str]]
    ] = Property(Array(AnyOf(ListAnyOfItem0, ListAnyOfItem1, String())))



from typing import List

from statham.dsl.constants import Maybe
from statham.dsl.elements import (
    Array,
    Boolean,
    Integer,
    Null,
    Number,
    Object,
    String,
)
from statham.dsl.property import Property


class Model(Object):

    string_no_validation: Maybe[str] = Property(String())

    string_default: str = Property(String(default="foo"))

    string_format_uuid: Maybe[str] = Property(String(format="uuid"))

    string_format_date_time: Maybe[str] = Property(String(format="date-time"))

    string_format_uri: Maybe[str] = Property(String(format="uri"))

    string_pattern: Maybe[str] = Property(String(pattern="^(foo|bar).*"))

    string_minLength: Maybe[str] = Property(String(minLength=3))

    string_maxLength: Maybe[str] = Property(String(maxLength=3))

    integer_no_validation: Maybe[int] = Property(Integer())

    integer_default: int = Property(Integer(default=1))

    integer_minimum: Maybe[int] = Property(Integer(minimum=3))

    integer_exclusiveMinimum: Maybe[int] = Property(Integer(exclusiveMinimum=3))

    integer_maximum: Maybe[int] = Property(Integer(maximum=3))

    integer_exclusiveMaximum: Maybe[int] = Property(Integer(exclusiveMaximum=3))

    integer_multipleOf: Maybe[int] = Property(Integer(multipleOf=2))

    number_no_validation: Maybe[float] = Property(Number())

    number_default: float = Property(Number(default=1.5))

    number_minimum: Maybe[float] = Property(Number(minimum=2.5))

    number_exclusiveMinimum: Maybe[float] = Property(
        Number(exclusiveMinimum=2.5)
    )

    number_maximum: Maybe[float] = Property(Number(maximum=2.5))

    number_exclusiveMaximum: Maybe[float] = Property(
        Number(exclusiveMaximum=2.5)
    )

    number_multipleOf: Maybe[float] = Property(Number(multipleOf=2.5))

    boolean_no_validation: Maybe[bool] = Property(Boolean())

    boolean_default: bool = Property(Boolean(default=True))

    null_no_validation: Maybe[None] = Property(Null())

    array_no_validation: Maybe[List[int]] = Property(Array(Integer()))

    array_minItems: Maybe[List[int]] = Property(Array(Integer(), minItems=1))

    array_maxItems: Maybe[List[int]] = Property(Array(Integer(), maxItems=2))
